#!/bin/sh
#
# script for showing more readable version of the sp-timestamp-strace
# output.
#
# This file is part of sp-timestamp.
#
# Copyright (C) 2009 Nokia Corporation.
#
# Contact: Eero Tamminen <eero.tamminen@nokia.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
# 02110-1301 USA

usage ()
{
   echo
   echo "Usage: ${0##*/} <strace output file>"
   echo
   echo "ERROR: $1!"
   echo
   exit 1
}

# arg sanity checking
if [ $# -ne 1 ]; then
	usage "File name missing"
fi
if [ \! -f $1 ]; then
	usage "File '$1' doesn't exist"
fi

# show readable version of the file:
# - time stamps with program name and message
# - succesful open calls with filename and FD
# - clone calls with the new TID/PID
# - execve calls with program name
egrep \
  -e 'open\("/tmp/stamps/' \
  -e 'open\(.*\) = [0-9]' \
  -e 'clone\(.*\) = [0-9]' \
  -e 'execve\(' $1 \
| sed \
  -e 's%^\(.*\) open("/tmp/stamps/\([^:]*:\)\([^"]*\)".*$%\1 \2 \3%' \
  -e 's%^\(.* open("[^"]*"\).*\() = [0-9]*\).*$%\1\2%' \
  -e 's%^\(.* clone(\).*\() = [0-9]*\).*$%\1\2%' \
  -e 's%^\(.* execve("[^"]*"\).*$%\1)%' \
