#!/bin/sh
#
# try to strace given PIDs or run the given executable with strace using
# arguments approriate for the plot-timeline.py visualization script.
#
# This file is part of sp-timestamp.
#
# Copyright (C) 2008,2009 Nokia Corporation.
#
# Contact: Eero Tamminen <eero.tamminen@nokia.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
# 02110-1301 USA
#
# Changes:
# 2009-10-06:
# - rewrite to support giving both PID(s) and executable at the same time
# - support maemo-launched applications
# - remove plot-timeline calling
# 2009-10-19:
# - filename needs to be specified for strace, otherwise output file
#   has stuff that messes up the post-processing

usage ()
{
   echo
   echo "Usage: ${0##*/} <output file> [PID1 PID2 ...] [program [options]]"
   echo
   echo "ERROR: $1!"
   echo
   exit 1
}

if [ -z "$(which strace)" ]; then
	echo "ERROR: 'strace' utility missing."
	exit 1
fi

if [ $# -lt 2 ]; then
	usage "too few arguments"
	exit 1
fi

log=$1
shift

pids=""
# parse PIDs
for pid in $*; do
	if [ \! -z "$(echo $pid|sed 's/[0-9]*//')" ]; then
		# PIDs done, arg has other stuff besides numbers
		break
	fi
	shift
	if [ -d /proc/$pid/ ]; then
		pids="$pids -p $pid"
	else
		usage "There's no process with PID $pid"
	fi
done

if [ \! -z "$1" ]; then
	if [ -x $1 ]; then
		app=$1
	else
		app=$(which $1)
		if [ $? != 0 ]; then
			usage "Given program '$1' doesn't exist or isn't executable"
		fi
	fi
	shift
	args="$*"
	
	if [ -e $app.launch ]; then
		echo "Maemo-launched application, attaching to maemo-launcher..."
		if [ \! -z "$args" ]; then
			echo "WARNING: Maemo launched applications usually don't take arguments."
		fi
		launcher=$(pidof maemo-launcher|tr ' ' '\n'|tail -1)
		pids="$pids -p $launcher"
		app=""
	fi
fi

if [ -z "$pids" ] && [ -z "$app" ]; then
	usage "You need to specify either PIDs or a program"
elif [ \! -z "$pids" ] && [ \! -z "$app" ]; then
	echo "ERROR: current strace supports only PIDs *or* program, see:"
	echo "  http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=549942"
	exit 1
fi

straceopts="-o $log -f -e trace=open,clone,execve -ttt"
echo strace $straceopts $pids $app $args
strace $straceopts $pids $app $args
