/*
 * Part of sp-timestamp.
 * 
 * Copyright (C) 2008-2009 by Nokia Corporation
 *
 * Contact: Eero Tamminen <eero.tamminen@nokia.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <string.h>
#include <assert.h>

#define SP_TIMESTAMP_CREATE 1
#define SP_TIMESTAMP_EXTERN 1
#include "sp_timestamp.h"

#define MESSAGE_MAX 255

/* intialize filename and offset variables */
static int sp_timestamp_init(char *filename, int maxlen)
{
	int fd, offset, count;
	char *cmdname;

	/* get process command name */
	fd = open("/proc/self/cmdline", O_RDONLY);
	assert(fd >= 0);

	offset = strlen(filename);
	count = read(fd, filename+offset, maxlen-offset);
	filename[maxlen] = '\0';
	assert(count > 0);
	close(fd);

	/* use basename of the command name */
	cmdname = strrchr(filename+offset, '/');
	memmove(filename+offset, cmdname+1, strlen(cmdname));
	offset += strlen(filename+offset);
	filename[offset++] = ':';
	
	return offset;
}

void sp_timestamp(const char *step)
{
	static char filename[MESSAGE_MAX] = SP_TIMESTAMP_PATH;
	static int offset, maxlen = sizeof(filename)-1;
	int fd;

	if (!offset) {
		offset = sp_timestamp_init(filename, maxlen);
	}
	strncpy(filename+offset, step, maxlen-offset);
	filename[maxlen] = '\0';
	
	fd = open(filename, O_CREAT|O_WRONLY, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
	if (fd >= 0) {
		close(fd);
	}
}
