#!/bin/sh
set -e

if [ -z "$(which strace)" ]; then
	echo "ERROR: 'strace' utility missing."
	exit 1
fi

sbconf=/targets/links/scratchbox.config
if [ -f $sbconf ]; then
	grep CPUTRANSPARENCY $sbconf|grep -qi qemu
	if [ $? -eq 0 ]; then
		echo "ERROR: Running in Scratchbox under Qemu."
		echo "Qemu doesn't support ptrace(2) needed by strace(1)."
		exit 1
	fi
fi

if [ -z "$(which plot-timeline)" ] || [ -z "$(which display)" ]; then
	echo "------------------------------------------------------------"
	echo "WARNING: 'plot-timeline' and/or 'display' utilities missing."
	echo "Install 'sp-timestamp-visualize' and 'imagemagick' packages"
	echo "before running this script to see the graphs. (run on x86)"
	echo "----------------------------------------------------------"
	show_graph=no
else
	show_graph=yes
fi

# test all versions
for prog in timestamps-*; do
	if [ \! -x $prog ]; then
		echo "ERROR: '$prog' test missing!"
		exit 1
	fi
	echo "TEST: $prog"
	../scripts/sp-timestamp-strace.sh timestamps.log ./$prog
	../scripts/sp-timestamp-show.sh timestamps.log
	if [ $show_graph = yes ]; then
		plot-timeline -o timestamps.png timestamps.log
		if [ $? == 0 ] && [ -f timestamps.png ]; then
			display timestamps.png
		fi
	fi
done

rm -f timestamps.log timestamps.png
